@@ -1,4 +1,4 @@
-// Copyright (C) 2024, Benjamin Drung <bdrung@posteo.de>
+// Copyright (C) 2024-2026, Benjamin Drung <bdrung@posteo.de>
 // SPDX-License-Identifier: ISC
 
 use std::collections::HashMap;
@@ -74,7 +74,9 @@ impl UserGroupCache {
         match self.user_cache.get(&uid) {
             Some(name) => Ok(name.clone()),
             None => {
-                let name = libc::getpwuid_name(uid)?;
+                let name = libc::getpwuid_name(uid).map_err(|e| {
+                    std::io::Error::new(e.kind(), format!("Get name for user ID {uid} failed: {e}"))
+                })?;
                 self.user_cache.insert(uid, name.clone());
                 Ok(name)
             }
@@ -86,7 +88,12 @@ impl UserGroupCache {
         match self.group_cache.get(&gid) {
             Some(name) => Ok(name.clone()),
             None => {
-                let name = libc::getgrgid_name(gid)?;
+                let name = libc::getgrgid_name(gid).map_err(|e| {
+                    std::io::Error::new(
+                        e.kind(),
+                        format!("Get name for group ID {gid} failed: {e}"),
+                    )
+                })?;
                 self.group_cache.insert(gid, name.clone());
                 Ok(name)
             }
@@ -370,6 +377,13 @@ mod tests {
     // Lock for tests that rely on / change the current directory
     pub(crate) static TEST_LOCK: std::sync::Mutex<u32> = std::sync::Mutex::new(0);
 
+    pub(crate) fn set_timezone_to_utc() {
+        unsafe {
+            env::set_var("TZ", "UTC");
+            tzset();
+        }
+    }
+
     pub(crate) fn tests_path<P: AsRef<Path>>(path: P) -> PathBuf {
         Path::new(env!("CARGO_MANIFEST_DIR"))
             .join("tests")
@@ -417,7 +431,7 @@ mod tests {
         .unwrap();
         assert_eq!(
             String::from_utf8(output).unwrap(),
-            ".\nusr\nusr/bin\nusr/bin/sh\n"
+            ".\nusr\nusr/bin\nusr/bin/ash\nusr/bin/sh\n"
         );
     }
 
@@ -450,8 +464,7 @@ mod tests {
         TRAILER!!!\0\0\0\0";
         let mut output = Vec::new();
         let mut user_group_cache = UserGroupCache::new();
-        env::set_var("TZ", "UTC");
-        unsafe { tzset() };
+        set_timezone_to_utc();
         read_cpio_and_print_long_format(
             &mut archive.as_ref(),
             &mut output,
@@ -479,8 +492,7 @@ mod tests {
         let mut output = Vec::new();
         let mut user_group_cache = UserGroupCache::new();
         user_group_cache.insert_test_data();
-        env::set_var("TZ", "UTC");
-        unsafe { tzset() };
+        set_timezone_to_utc();
         read_cpio_and_print_long_format(
             &mut archive.as_ref(),
             &mut output,
@@ -509,8 +521,7 @@ mod tests {
         let mut output = Vec::new();
         let mut user_group_cache = UserGroupCache::new();
         user_group_cache.insert_test_data();
-        env::set_var("TZ", "UTC");
-        unsafe { tzset() };
+        set_timezone_to_utc();
         read_cpio_and_print_long_format(
             &mut archive.as_ref(),
             &mut output,
@@ -542,8 +553,7 @@ mod tests {
         let mut output = Vec::new();
         let mut user_group_cache = UserGroupCache::new();
         user_group_cache.insert_test_data();
-        env::set_var("TZ", "UTC");
-        unsafe { tzset() };
+        set_timezone_to_utc();
         read_cpio_and_print_long_format(
             &mut archive.as_ref(),
             &mut output,
@@ -598,8 +608,7 @@ mod tests {
         let mut output = Vec::new();
         let mut user_group_cache = UserGroupCache::new();
         user_group_cache.insert_test_data();
-        env::set_var("TZ", "UTC");
-        unsafe { tzset() };
+        set_timezone_to_utc();
         read_cpio_and_print_long_format(
             &mut archive.as_ref(),
             &mut output,
